/* $XConsortium: OthelloDisc.c,v 1.79 94/04/17 20:11:58 kaleb Exp $ */

/***********************************************************

Copyright (c) 1987, 1988, 1994  X Consortium

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Except as contained in this notice, the name of the X Consortium shall not be
used in advertising or otherwise to promote the sale, use or other dealings
in this Software without prior written authorization from the X Consortium.


Copyright 1987, 1988 by Digital Equipment Corporation, Maynard, Massachusetts.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the name of Digital not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

******************************************************************/

/*
 * OthelloDisc.c - OthelloDisc button widget
 */

#include <stdio.h>
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include <X11/Xmu/Misc.h>
#include <X11/Xaw3d/XawInit.h>
#include "OthelloDisc/OthelloDiscP.h"
#include <X11/Xmu/Converters.h>
#include <X11/Xaw3d/ThreeDP.h>

#define DEFAULT_HIGHLIGHT_THICKNESS 2

/****************************************************************
 *
 * Full class record constant
 *
 ****************************************************************/

/* Private Data */

static char defaultTranslations[] =
    "<EnterWindow>:	highlight()		\n\
     <LeaveWindow>:	reset()			\n\
     <Btn1Down>:	set()			\n\
     <Btn1Up>:		notify() unset()	";

#define offset(field) XtOffsetOf(OthelloDiscRec, field)
static XtResource resources[] = { 
   {XtNcallback, XtCCallback, XtRCallback, sizeof(XtPointer), 
      offset(othelloDisc.callbacks), XtRCallback, (XtPointer)NULL},

   {XtNenterCallback, XtCCallback, XtRCallback, sizeof(XtPointer), 
      offset(othelloDisc.enter_callbacks), XtRCallback, (XtPointer)NULL},
   {XtNleaveCallback, XtCCallback, XtRCallback, sizeof(XtPointer), 
      offset(othelloDisc.leave_callbacks), XtRCallback, (XtPointer)NULL},

   {XtNhighlightThickness, XtCThickness, XtRDimension, sizeof(Dimension),
      offset(othelloDisc.highlight_thickness), XtRImmediate,
      (XtPointer) 2},

   {XtNforeground, XtCForeground, XtRPixel, sizeof(Pixel),
      offset(othelloDisc.foreground), XtRString, XtDefaultForeground },
   {XtNborderColor, XtCBackground, XtRPixel, sizeof(Pixel),
      offset(othelloDisc.border_color), XtRString, XtDefaultBackground },
   {XtNblackColor, XtCForeground, XtRPixel, sizeof(Pixel),
      offset(othelloDisc.black_color), XtRString, "black" },
   {XtNwhiteColor, XtCForeground, XtRPixel, sizeof(Pixel),
      offset(othelloDisc.white_color), XtRString, "white" },
   {XtNmiddleHighlightColor, XtCForeground, XtRPixel, sizeof(Pixel),
      offset(othelloDisc.middle_color), XtRString, "gray" },
   {XtNlowHighlightColor, XtCForeground, XtRPixel, sizeof(Pixel),
      offset(othelloDisc.low_color), XtRString, "gray50" },

   {XtNshadowWidth, XtCShadowWidth, XtRDimension, sizeof(Dimension),
      offset(threeD.shadow_width), XtRImmediate, (XtPointer) 2},
   {XtNborderWidth, XtCBorderWidth, XtRDimension, sizeof(Dimension),
      XtOffsetOf(RectObjRec,rectangle.border_width), XtRImmediate,
      (XtPointer)0},

   {XtNstate, XtCState, XtRInt, sizeof(int),
      offset(othelloDisc.state), XtRImmediate, (XtPointer) 0},
   {XtNborderState, XtCState, XtRInt, sizeof(int),
      offset(othelloDisc.border_state), XtRImmediate, (XtPointer) ODHighlightNone},
   {XtNmarkState, XtCState, XtRInt, sizeof(int),
      offset(othelloDisc.mark_state), XtRImmediate, (XtPointer) ODMarkNone},
   {XtNflipState, XtCState, XtRInt, sizeof(int),
      offset(othelloDisc.flip_state), XtRImmediate, (XtPointer) 0},
   {XtNflipOrientation, XtCOrientation, XtROrientation, sizeof(XtOrientation),
      offset(othelloDisc.flip_orientation), XtRImmediate, (XtPointer) XtorientHorizontal},
};
#undef offset

static Boolean SetValues();
static void Initialize(), Redisplay(), Set(), Reset(), Notify(), Unset();
static void Highlight(), Unhighlight(), Destroy(), PaintOthelloDiscWidget();
static void ClassInitialize();
static void Realize(), Resize();

static XtActionsRec actionsList[] = {
  {"set",		Set},
  {"notify",		Notify},
  {"highlight",		Highlight},
  {"reset",		Reset},
  {"unset",		Unset},
  {"unhighlight",	Unhighlight}
};

#define SuperClass ((SimpleWidgetClass)&simpleClassRec)

OthelloDiscClassRec othelloDiscClassRec = {
  {
    (WidgetClass) &threeDClassRec,		/* superclass		  */	
    "OthelloDisc",				/* class_name		  */
    sizeof(OthelloDiscRec),			/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    Realize,				/* realize		  */
    actionsList,			/* actions		  */
    XtNumber(actionsList),		/* num_actions		  */
    resources,				/* resources		  */
    XtNumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    FALSE,				/* compress_motion	  */
    TRUE,				/* compress_exposure	  */
    TRUE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    Destroy,				/* destroy		  */
    Resize,				/* resize		  */
    Redisplay,				/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    defaultTranslations,		/* tm_table		  */
    XtInheritQueryGeometry,		/* query_geometry	  */
    XtInheritDisplayAccelerator,	/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
    XtInheritChangeSensitive		/* change_sensitive	*/
  },  /* SimpleClass fields initialization */
  {
    XtInheritXaw3dShadowDraw,           /* shadowdraw           */
  },  /* ThreeD Class fields initialization */
  {
    0,                                     /* field not used    */
  },  /* OthelloDiscClass fields initialization */
};

/* for public consumption */
WidgetClass othelloDiscWidgetClass = (WidgetClass) &othelloDiscClassRec;

/****************************************************************
 *
 * Private Procedures
 *
 ****************************************************************/

static GC 
Get_GC(cbw, fg, bg, flag)
OthelloDiscWidget cbw;
Pixel fg, bg;
Boolean flag; /* for border */
{
  XGCValues	values;
  
  values.foreground   = fg;
  values.background	= bg;
  values.cap_style = CapProjecting;
  
  if (flag) {
    if (cbw->othelloDisc.highlight_thickness > 1 )
      values.line_width   = cbw->othelloDisc.highlight_thickness;
    else 
      values.line_width   = 0;
  } else {
    values.line_width = 1;
  }
  values.fill_rule = WindingRule;
  
  return XtGetGC((Widget)cbw, 
		 (GCForeground|GCBackground|GCLineWidth|GCCapStyle|GCFillRule),
		 &values);
}


/* ARGSUSED */
static void 
Initialize(request, new, args, num_args)
Widget request, new;
ArgList args;			/* unused */
Cardinal *num_args;		/* unused */
{
  OthelloDiscWidget cbw = (OthelloDiscWidget) new;

  cbw->othelloDisc.normal_GC = Get_GC(cbw, cbw->othelloDisc.foreground, 
				      cbw->core.background_pixel, True);
  cbw->othelloDisc.inverse_GC = Get_GC(cbw, cbw->core.background_pixel, 
				       cbw->othelloDisc.foreground, True);
  cbw->othelloDisc.middle_GC = Get_GC(cbw, cbw->othelloDisc.middle_color,
				      cbw->core.background_pixel, True);
  cbw->othelloDisc.low_GC = Get_GC(cbw, cbw->othelloDisc.low_color,
				   cbw->core.background_pixel, True);
  cbw->othelloDisc.border_GC = Get_GC(cbw, cbw->othelloDisc.border_color,
				      cbw->core.background_pixel, True);
  cbw->othelloDisc.black_GC = Get_GC(cbw, cbw->othelloDisc.black_color,
				     cbw->core.background_pixel, False);
  cbw->othelloDisc.white_GC = Get_GC(cbw, cbw->othelloDisc.white_color,
				     cbw->core.background_pixel, False);
				     
  cbw->othelloDisc.set = FALSE;
  cbw->othelloDisc.highlighted = ODHighlightNone;
}

static Region 
HighlightRegion(cbw)
OthelloDiscWidget cbw;
{
  static Region outerRegion = NULL, innerRegion, emptyRegion;
  Dimension s = cbw->threeD.shadow_width;
  XRectangle rect;

  if (cbw->othelloDisc.highlight_thickness == 0 ||
      cbw->othelloDisc.highlight_thickness >
      (Dimension) ((Dimension) Min(cbw->core.width, cbw->core.height)/2))
    return(NULL);

  if (outerRegion == NULL) {
    /* save time by allocating scratch regions only once. */
    outerRegion = XCreateRegion();
    innerRegion = XCreateRegion();
    emptyRegion = XCreateRegion();
  }

  rect.x = rect.y = s;
  rect.width = cbw->core.width - 2 * s;
  rect.height = cbw->core.height - 2 * s;
  XUnionRectWithRegion( &rect, emptyRegion, outerRegion );
  rect.x = rect.y += cbw->othelloDisc.highlight_thickness;
  rect.width -= cbw->othelloDisc.highlight_thickness * 2;
  rect.height -= cbw->othelloDisc.highlight_thickness * 2;
  XUnionRectWithRegion( &rect, emptyRegion, innerRegion );
  XSubtractRegion( outerRegion, innerRegion, outerRegion );
  return outerRegion;
}

/***************************
*
*  Action Procedures
*
***************************/

/* ARGSUSED */
static void 
Set(w,event,params,num_params)
Widget w;
XEvent *event;
String *params;		/* unused */
Cardinal *num_params;	/* unused */
{
  OthelloDiscWidget cbw = (OthelloDiscWidget)w;

  if (cbw->othelloDisc.set)
    return;

  cbw->othelloDisc.set= TRUE;
  if (XtIsRealized(w))
    PaintOthelloDiscWidget(w, event, (Region) NULL, TRUE);
}

/* ARGSUSED */
static void
Unset(w,event,params,num_params)
Widget w;
XEvent *event;
String *params;		/* unused */
Cardinal *num_params;
{
  OthelloDiscWidget cbw = (OthelloDiscWidget)w;

  if (!cbw->othelloDisc.set)
    return;

  cbw->othelloDisc.set = FALSE;
  if (XtIsRealized(w)) {
    XClearWindow(XtDisplay(w), XtWindow(w));
    PaintOthelloDiscWidget(w, event, (Region) NULL, TRUE);
  }
}

/* ARGSUSED */
static void 
Reset(w,event,params,num_params)
Widget w;
XEvent *event;
String *params;		/* unused */
Cardinal *num_params;   /* unused */
{
  OthelloDiscWidget cbw = (OthelloDiscWidget)w;

  if (cbw->othelloDisc.set) {
    cbw->othelloDisc.highlighted = ODHighlightNone;
    Unset(w, event, params, num_params);
  } else
    Unhighlight(w, event, params, num_params);
}

/* ARGSUSED */
static void 
Highlight(w,event,params,num_params)
Widget w;
XEvent *event;
String *params;		
Cardinal *num_params;	
{
  OthelloDiscWidget cbw = (OthelloDiscWidget)w;

  cbw->othelloDisc.highlighted = ODHighlightHigh;

  if (XtIsRealized(w))
    PaintOthelloDiscWidget(w, event, HighlightRegion(cbw), TRUE);

  XtCallCallbackList(w, cbw->othelloDisc.enter_callbacks, (XtPointer) NULL);

}

/* ARGSUSED */
static void 
Unhighlight(w,event,params,num_params)
Widget w;
XEvent *event;
String *params;		/* unused */
Cardinal *num_params;	/* unused */
{
  OthelloDiscWidget cbw = (OthelloDiscWidget)w;

  cbw->othelloDisc.highlighted = ODHighlightNone;
  if (XtIsRealized(w))
    PaintOthelloDiscWidget(w, event, HighlightRegion(cbw), TRUE);

  XtCallCallbackList(w, cbw->othelloDisc.leave_callbacks, (XtPointer) NULL);
}

/* ARGSUSED */
static void 
Notify(w,event,params,num_params)
Widget w;
XEvent *event;
String *params;		/* unused */
Cardinal *num_params;	/* unused */
{
  OthelloDiscWidget cbw = (OthelloDiscWidget)w; 

  /* check to be sure state is still Set so that user can cancel
     the action (e.g. by moving outside the window, in the default
     bindings.
  */
  if (cbw->othelloDisc.set)
    XtCallCallbackList(w, cbw->othelloDisc.callbacks, (XtPointer) NULL);
}

/*
 * Repaint the widget window
 */

/************************
*
*  REDISPLAY (DRAW)
*
************************/

/* ARGSUSED */
static void 
Redisplay(w, event, region)
Widget w;
XEvent *event;
Region region;
{
  PaintOthelloDiscWidget(w, event, region, FALSE);
}

/*	Function Name: PaintOthelloDiscWidget
 *	Description: Paints the OthelloDisc widget.
 *	Arguments: w - the OthelloDisc widget.
 *                 region - region to paint (passed to the superclass).
 *                 change - did it change either set or highlight state?
 *	Returns: none
 */

static void 
PaintOthelloDiscWidget(w, event, region, change)
Widget w;
XEvent *event;
Region region;
Boolean change;
{
  OthelloDiscWidget cbw = (OthelloDiscWidget) w;
  OthelloDiscWidgetClass cwclass = (OthelloDiscWidgetClass) XtClass (w);

  GC border_gc;
  GC disc_gc, discr_gc;

  Dimension s = cbw->threeD.shadow_width;
  Dimension x = cbw->core.width / 2;
  Dimension y = cbw->core.height / 2;
  Dimension rx, ry, dy; /* disc size, disc thick */

  int f; /* flip status (0-7) */

  /*  (*SuperClass->core_class.expose) (w, event, region);*/

  /* Border */

  border_gc = cbw->othelloDisc.inverse_GC;
  if (cbw->othelloDisc.highlight_thickness > 0) {
    /* draw border */
    switch (cbw->othelloDisc.border_state) {
    case ODHighlightHigh:
      border_gc = cbw->othelloDisc.normal_GC;
      break;
    case ODHighlightMiddle:
      border_gc = cbw->othelloDisc.middle_GC;
      break;
    case ODHighlightLow:
      border_gc = cbw->othelloDisc.low_GC;
      break;
    case ODHighlightNone:
      border_gc = cbw->othelloDisc.border_GC;
      break;
    } 
    if (cbw->othelloDisc.highlighted) {
      border_gc = cbw->othelloDisc.normal_GC;
    }
    
    if (!cbw->othelloDisc.set) {
      int offset = cbw->othelloDisc.highlight_thickness/2;
      XDrawRectangle(XtDisplay(w),XtWindow(w), border_gc, s + offset, s + offset, 
		     cbw->core.width - cbw->othelloDisc.highlight_thickness - 2 * s,
		     cbw->core.height - cbw->othelloDisc.highlight_thickness - 2 * s);
    }
  }

  /* Disc */

  f = cbw->othelloDisc.flip_state;
  if (f > 7) f = 15 - f;
  if (f < 0) f = 0;

  if (cbw->othelloDisc.flip_orientation == XtorientHorizontal) {
    rx = x * (8 - f) / 10;
    ry = y * 8 / 11;
  } else {
    rx = x * 8 / 10;
    ry = y * (8 - f) / 11;
  }
  dy = 1 + ry / 17;

  if (cbw->othelloDisc.state) {
    if ((cbw->othelloDisc.state == 1) ^ (cbw->othelloDisc.flip_state > 7)) {
      disc_gc = cbw->othelloDisc.black_GC;
      discr_gc = cbw->othelloDisc.white_GC;
    } else {
      disc_gc = cbw->othelloDisc.white_GC;
      discr_gc = cbw->othelloDisc.black_GC;
    }

    XFillArc(XtDisplay(w), XtWindow(w), discr_gc, x - rx, y - ry + dy, rx * 2, ry * 2, 0, 360*64);
    XFillArc(XtDisplay(w), XtWindow(w), disc_gc, x - rx, y - ry - dy, rx * 2, ry * 2, 0, 360*64);
  } else {
    disc_gc = cbw->othelloDisc.black_GC;
    discr_gc = cbw->othelloDisc.white_GC;
    f = 0; // setup to draw mark with White pen
  }
  
  /* mark */

  if (cbw->othelloDisc.mark_state && !f) {
    int x1, x2, x3, x4, y1, y2, y3, u;
    XPoint p[5];

    u = ((rx > ry) ? ry : rx) * 100 / 3;
    if (u < 200) u = 200; /* hundred units of drawing */

    switch(cbw->othelloDisc.mark_state) {
    case ODMarkNone:
      break;
    case ODMarkStar:
      x1 = x - u / 100; x2 = x - u / 140; x3 = x + u / 140; x4 = x + u / 100;
      y1 = y - u / 100; y2 = y - u / 300; y3 = y + u / 110;
      /*
	 XDrawLine(XtDisplay(w), XtWindow(w), discr_gc, x1, y2, x4, y2);
	 XDrawLine(XtDisplay(w), XtWindow(w), discr_gc, x1, y2, x3, y3);
	 XDrawLine(XtDisplay(w), XtWindow(w), discr_gc, x4, y2, x2, y3);
	 XDrawLine(XtDisplay(w), XtWindow(w), discr_gc, x, y1, x2, y3);
	 XDrawLine(XtDisplay(w), XtWindow(w), discr_gc, x, y1, x3, y3); */
      p[0].x = x; p[1].x = x3; p[2].x = x1 + 1; p[3].x = x4; p[4].x = x2;
      p[0].y = y1; p[1].y = p[4].y = y3; p[2].y = p[3].y = y2;
      XFillPolygon(XtDisplay(w), XtWindow(w), discr_gc, p, 5, Complex, CoordModeOrigin);
      break;
    case ODMarkPlus:
      x1 = x - u / 100; x2 = x + u / 100; 
      y1 = y - u / 100; y2 = y +  u / 100; 
      XDrawLine(XtDisplay(w), XtWindow(w), discr_gc, x, y1, x, y2);
      XDrawLine(XtDisplay(w), XtWindow(w), discr_gc, x1, y, x2, y);
      break;
    case ODMarkCircle:
      x1 = u / 200; /* radius */
      XDrawArc(XtDisplay(w), XtWindow(w), discr_gc, x, y, x1, x1, 0, 360 * 64);
      break;
    }
  }
  (*cwclass->threeD_class.shadowdraw) (w, event, region, !cbw->othelloDisc.set);
}

static void 
Destroy(w)
Widget w;
{
  OthelloDiscWidget cbw = (OthelloDiscWidget) w;

  XtReleaseGC( w, cbw->othelloDisc.inverse_GC );
  XtReleaseGC( w, cbw->othelloDisc.normal_GC );
  XtReleaseGC( w, cbw->othelloDisc.black_GC );
  XtReleaseGC( w, cbw->othelloDisc.white_GC );
  XtReleaseGC( w, cbw->othelloDisc.middle_GC );
  XtReleaseGC( w, cbw->othelloDisc.low_GC );
  XtReleaseGC( w, cbw->othelloDisc.border_GC );
}

/*
 * Set specified arguments into widget
 */

/* ARGSUSED */
static Boolean 
SetValues (current, request, new, args, num_args)
Widget current, request, new;
ArgList args;
Cardinal *num_args;
{
  OthelloDiscWidget oldcbw = (OthelloDiscWidget) current;
  OthelloDiscWidget cbw = (OthelloDiscWidget) new;
  Boolean redisplay = False;

  if ( oldcbw->core.sensitive != cbw->core.sensitive && !cbw->core.sensitive) {
    /* about to become insensitive */
    if (cbw->othelloDisc.set || cbw->othelloDisc.highlighted) redisplay = TRUE;
    cbw->othelloDisc.set = FALSE;
    cbw->othelloDisc.highlighted = ODHighlightNone;
  }
  
  if ( (oldcbw->othelloDisc.foreground != cbw->othelloDisc.foreground) ||
      (oldcbw->core.background_pixel != cbw->core.background_pixel) ||
      (oldcbw->othelloDisc.highlight_thickness != 
       cbw->othelloDisc.highlight_thickness)) {
    XtReleaseGC(new, cbw->othelloDisc.inverse_GC);
    XtReleaseGC(new, cbw->othelloDisc.normal_GC);
    
    cbw->othelloDisc.normal_GC = Get_GC(cbw, cbw->othelloDisc.foreground, 
					cbw->core.background_pixel);
    cbw->othelloDisc.inverse_GC = Get_GC(cbw, cbw->core.background_pixel, 
					 cbw->othelloDisc.foreground);
    
    redisplay = True;
  }

  if ( (oldcbw->othelloDisc.black_color != cbw->othelloDisc.black_color) ) {
    XtReleaseGC(new, cbw->othelloDisc.black_GC);
    cbw->othelloDisc.black_GC = Get_GC(cbw, cbw->othelloDisc.black_color,
				       cbw->core.background_pixel);
    redisplay = True;
  }
  if ( (oldcbw->othelloDisc.white_color != cbw->othelloDisc.white_color) ) {
    XtReleaseGC(new, cbw->othelloDisc.white_GC);
    cbw->othelloDisc.white_GC = Get_GC(cbw, cbw->othelloDisc.white_color,
				       cbw->core.background_pixel);
    redisplay = True;
  }
 if ( (oldcbw->othelloDisc.middle_color != cbw->othelloDisc.middle_color) ) {
    XtReleaseGC(new, cbw->othelloDisc.middle_GC);
    cbw->othelloDisc.middle_GC = Get_GC(cbw, cbw->othelloDisc.middle_color,
					cbw->core.background_pixel);
    redisplay = True;
  }
  if ( (oldcbw->othelloDisc.low_color != cbw->othelloDisc.low_color) ) {
    XtReleaseGC(new, cbw->othelloDisc.low_GC);
    cbw->othelloDisc.low_GC = Get_GC(cbw, cbw->othelloDisc.low_color,
				     cbw->core.background_pixel);
    redisplay = True;
  }
  if ( (oldcbw->othelloDisc.border_color != cbw->othelloDisc.border_color) ) {
    XtReleaseGC(new, cbw->othelloDisc.border_GC);
    cbw->othelloDisc.border_GC = Get_GC(cbw, cbw->othelloDisc.border_color,
				     cbw->core.background_pixel);
    redisplay = True;
  }
  if ( (oldcbw->othelloDisc.state != cbw->othelloDisc.state) ) {
    redisplay = True;
  }
  if ( (oldcbw->othelloDisc.border_state != cbw->othelloDisc.border_state) ) {
    redisplay = True;
  }
  if ( (oldcbw->othelloDisc.mark_state != cbw->othelloDisc.mark_state) ) {
    redisplay = True;
  }
  if (cbw->othelloDisc.state) {
    int oldf = oldcbw->othelloDisc.flip_state;
    int newf = cbw->othelloDisc.flip_state;
    if (oldf > 15) oldf = 15; else if (oldf < 0) oldf = 0;
    if (newf > 15) newf = 15; else if (newf < 0) newf = 0;
    if ( oldf != newf ) {
      redisplay = True;
    }
  }
  return (redisplay);
}

static void ClassInitialize()
{
    XawInitializeWidgetSet();
    XtAddConverter( XtRString, XtROrientation, XmuCvtStringToOrientation,
		   (XtConvertArgList)NULL, (Cardinal)0 );
}


static void Realize(w, valueMask, attributes)
    Widget w;
    Mask *valueMask;
    XSetWindowAttributes *attributes;
{
    (*othelloDiscWidgetClass->core_class.superclass->core_class.realize)
	(w, valueMask, attributes);
}

static void Resize(w)
    Widget w;
{

}
